import math
from datetime import datetime

number_start = 0

# Function to convert RGB to XYZ
def convert_rgb_to_xyz(r, g, b):
    # Normalize the RGB values (0-100 scale)
    r = r / 100
    g = g / 100
    b = b / 100

    # Linearize the RGB values
    r = math.pow((r + 0.055) / 1.055, 2.4) if r > 0.04045 else r / 12.92
    g = math.pow((g + 0.055) / 1.055, 2.4) if g > 0.04045 else g / 12.92
    b = math.pow((b + 0.055) / 1.055, 2.4) if b > 0.04045 else b / 12.92

    # Convert to XYZ using the sRGB color space D65 illuminant
    x = (r * 0.4124564 + g * 0.3575761 + b * 0.1804375)*100
    y = (r * 0.2126729 + g * 0.7151522 + b * 0.0721750)*100
    z = (r * 0.0193339 + g * 0.1191920 + b * 0.9503041)*100

    return round(x, 6), round(y, 6), round(z, 6)

# Prompt for input and output file paths
input_file = "input.txt"
output_file = "output.ti1"

# Begin the .ti1 file with strict formatting for CGATS compliance


with open(output_file, 'w') as file:
    file.write('CTI1\n')
    file.write(f'DESCRIPTOR "Argyll Calibration Target chart information 1"\n')
    file.write(f'ORIGINATOR "Argyll targen"\n')
    file.write(f'CREATED "{datetime.now().strftime("%a %b %d %H:%M:%S %Y")}"\n')
    file.write('APPROX_WHITE_POINT "95.106486 100.000000 108.844025"\n')
    file.write('COLOR_REP "iRGB"\n')
    file.write('WHITE_COLOR_PATCHES "4"\n')
    file.write('BLACK_COLOR_PATCHES "4"\n')
    file.write('OFPS_PATCHES "217"\n')
    file.write('\nNUMBER_OF_FIELDS 7\n')
    file.write('BEGIN_DATA_FORMAT\n')
    file.write('SAMPLE_ID RGB_R RGB_G RGB_B XYZ_X XYZ_Y XYZ_Z\n')
    file.write('END_DATA_FORMAT\n\n')

    # Initialize variables for the data section
    data_lines = []

    # Process RGB data and generate output lines
    with open(input_file, 'r') as input_data:
        for line_number, rgb_values in enumerate(input_data.readlines(), start=1):
            rgb = [int(value) for value in rgb_values.split()]
            # Scale RGB from 0-255 to 0-100
            r = round((rgb[0] / 255) * 100, 5)
            g = round((rgb[1] / 255) * 100, 5)
            b = round((rgb[2] / 255) * 100, 5)

            # Convert RGB to XYZ
            xyz_x, xyz_y, xyz_z = convert_rgb_to_xyz(r, g, b)
            
            write_number = int(line_number) + number_start

            # Format the data line
            data_lines.append(f"{str(write_number)} {r} {g} {b} {xyz_x} {xyz_y} {xyz_z}")

    # Write the main data block
    file.write(f'NUMBER_OF_SETS {len(data_lines)}\n')
    file.write('BEGIN_DATA\n')
    file.write('\n'.join(data_lines))
    file.write('\nEND_DATA\n')

    # Add DENSITY_EXTREME_VALUES section
    file.write('\nCTI1\n')
    file.write(f'DESCRIPTOR "Argyll Calibration Target chart information 1"\n')
    file.write(f'ORIGINATOR "Argyll targen"\n')
    file.write(f'DENSITY_EXTREME_VALUES "8"\n')
    file.write(f'CREATED "{datetime.now().strftime("%B %d, %Y")}"\n')
    file.write('\nNUMBER_OF_FIELDS 7\n')
    file.write('BEGIN_DATA_FORMAT\n')
    file.write('INDEX RGB_R RGB_G RGB_B XYZ_X XYZ_Y XYZ_Z\n')
    file.write('END_DATA_FORMAT\n\n')
    file.write('NUMBER_OF_SETS 8\n')
    file.write('BEGIN_DATA\n')
    # Example data for DENSITY_EXTREME_VALUES, replace with actual data if available
    file.write('0 100 100 100 95.1065 100 108.844\n')
    file.write('1 0.00000 47.3607 100 25.613 21.6292 97.3763\n')
    file.write('2 100 0.00000 79.3514 52.4258 26.2898 58.7215\n')
    file.write('3 0.00000 0.00000 58.9971 6.48583 3.19399 29.8945\n')
    file.write('4 100 66.6593 0.00000 56.0588 50.5054 7.65614\n')
    file.write('5 0.00000 35.6011 0.00000 4.68562 8.37021 2.22855\n')
    file.write('6 84.4444 0.00000 0.00000 28.8428 15.3558 2.30466\n')
    file.write('7 0.00000 0.00000 0.00000 1 1 1\n')
    file.write('END_DATA\n')

    # Add DEVICE_COMBINATION_VALUES section
    file.write('\nCTI1\n')
    file.write(f'DESCRIPTOR "Argyll Calibration Target chart information 1"\n')
    file.write(f'ORIGINATOR "Argyll targen"\n')
    file.write(f'DEVICE_COMBINATION_VALUES "9"\n')
    file.write(f'CREATED "{datetime.now().strftime("%B %d, %Y")}"\n')
    file.write('\nNUMBER_OF_FIELDS 7\n')
    file.write('BEGIN_DATA_FORMAT\n')
    file.write('INDEX RGB_R RGB_G RGB_B XYZ_X XYZ_Y XYZ_Z\n')
    file.write('END_DATA_FORMAT\n\n')
    file.write('NUMBER_OF_SETS 9\n')
    file.write('BEGIN_DATA\n')
    # Example data for DEVICE_COMBINATION_VALUES, replace with actual data if available
    file.write('0 100 100 100 95.1065 100 108.844\n')
    file.write('1 0.00000 100 100 54.2763 78.9478 106.931\n')
    file.write('2 100 0.00000 100 59.7013 29.1995 97.0422\n')
    file.write('3 0.00000 0.00000 100 18.8711 8.14731 95.129\n')
    file.write('4 100 100 0.00000 77.2354 92.8527 14.7151\n')
    file.write('5 0.00000 100 0.00000 36.4052 71.8005 12.8018\n')
    file.write('6 100 0.00000 0.00000 41.8302 22.0522 2.91323\n')
    file.write('7 0.00000 0.00000 0.00000 1 1 1\n')
    file.write('8 50 50 50 21.1427 22.1901 24.0831\n')
    file.write('END_DATA\n')